/* ------------------------------------------------------------------- */
/*                                                                     */
/*  Copyright (C)                                                      */ 
/*                                                                     */
/*  This program is free software; you can redistribute it and/or      */
/*  modify it under the terms of the GNU General Public License as     */
/*  published by the Free Software Foundation; either version 2 of the */
/*  License, or (at your option) any later version.                    */
/*                                                                     */
/*  This program is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of     */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   */
/*  General Public License for more details.                           */
/*                                                                     */
/*  You should have received a copy of the GNU General Public License  */
/*  along with this program; if not, write to the Free Software        */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,              */
/*  MA 02111-1307 USA                                                  */
/*                                                                     */
/*-------------------------------------------------------------------- */
/*  $Author: vogje02 $ $Revision: 1.7 $ $Date: 2002/07/28 17:13:19 $ */
/*---------------------------------------------------------------------*/

#ifndef __NMAP_SERV_H
#define __NMAP_SERV_H

#include <NTService.h>
#include <winperf.h>

#include <signal.h>
#include <LogWriter.h>

#define version "1.3.0"

// Service scan factors
#define INTERVAL_MINUTES  0
#define INTERVAL_HOURS    1
#define INTERVAL_DAYS     2
#define INTERVAL_WEEKS    3
#define INTERVAL_MONTHS   4

// Handy ascii -> bool conversion
#define atob(s) ( strncmp( s, "1", 1 ) == 0 ? true : false )

// Performance data
typedef PERF_DATA_BLOCK             PERF_DATA,      *PPERF_DATA;
typedef PERF_OBJECT_TYPE            PERF_OBJECT,    *PPERF_OBJECT;
typedef PERF_INSTANCE_DEFINITION    PERF_INSTANCE,  *PPERF_INSTANCE;
typedef PERF_COUNTER_DEFINITION     PERF_COUNTER,   *PPERF_COUNTER;

extern HKEY          hPerfKey = HKEY_PERFORMANCE_DATA;
extern HKEY		  hMachineKey = HKEY_LOCAL_MACHINE;			// get title index from this key

extern DWORD		  gpPerfDataSize = 200*1024;				// start with 200K
extern PPERF_DATA	  gpPerfData = NULL;

// Extern function
extern DWORD          PX_PROCESS;
extern DWORD          PX_PROCESS_ID;
extern DWORD          PX_THREAD;
extern TCHAR          INDEX_PROCTHRD_OBJ[20];
extern void           SetPerfIndexes();
extern int            ProcessGetPid( PPERF_OBJECT pObject, PPERF_INSTANCE pInstance );
extern int            ProcessGetNumThreads(  PPERF_DATA data, unsigned int ParentIndex  );
extern DWORD          GetPerfData( HKEY hPerfKey, LPTSTR szObjectIndex, 
								   PPERF_DATA *ppData, DWORD *pDataSize );
extern PPERF_OBJECT   FindObject( PPERF_DATA pData, DWORD TitleIndex );
extern PPERF_INSTANCE FirstInstance( PPERF_OBJECT pObject );
extern PPERF_INSTANCE NextInstance (PPERF_INSTANCE pInst);
extern LPTSTR         InstanceName (PPERF_INSTANCE pInst);

/**
 * NMap service class 
 */
class CNMapService : public CNTService
{
	public:
  
		/**
 		 * Constructor
		 */
		CNMapService( CLogWriter* lw );

		/**
		 * Run method for the underlying thread.
		 *
		 * This method is waiting for incoming requests and dispatches them.
		 */
		void RunService();

		/**
		 * Win32 service initialization
		 */
		BOOL OnInit();

		/**
		 * Called when service stops
		 */
		void OnStop();

		/**
		 * Process user control requests
		 *
		 * @param dwOpcode (DWORD) user control code
		 */ 
		BOOL OnUserControl( DWORD dwOpcode );

		/**
		 * Save the current status in the registry
		 */
		void SaveStatus();

		/**
		 * NT service or not
		 */
		BOOL bservice;

		/**
		 * NT service control parameter
		 */
		int m_iStartParam;

		/**
		 * NT service Control parameter
		 */
		int m_iIncParam;

		/**
		 * Current state
		 */
		int m_iState;
  
	public:

		/**
		 * Read the new values for the nmap and nmapserv arguments 
		 * from the registry.
		 */
		bool ReadRegistry();

		/**
		 * Parse the service arguments.
		 */
		void ParseServiceArgs();

		/**
		 * Get the scanning time.
		 */
		void GetNextScan();

		/**
		 * Start the nmap scanner. This is actually launching the nmap
		 * service as a background process.
		 */
		void StartScanner();

	private:

		/**
		 * Startup info of the service
		 */
		STARTUPINFO si;
		
		/**
		 * Process info of the service
		 */
		PROCESS_INFORMATION pi;

		/** 
		 * Returns the PID of the currently running process
		 */
        int GetPid( char* appl, int* nThreads );

		/**
		 * Log writer
		 */
		CLogWriter* logWriter;

		/**
		 * Registry key change event
		 */
		HANDLE hEvent;

		/**
		 * nmap arguments.
		 */
		unsigned char nmapArgs[1024];

		/**
		 * nmap arguments.
		 */
		unsigned char servArgs[1024];

		/**
		 * Scanning interval
		 */
		unsigned int interval;

		/**
		 * Scanning factor
		 */
		unsigned int factor;

		/**
		 * Scanning days
		 */
		bool days[7];

		/**
		 * Hour, min, sec of next scan.
		 */
		int hour, min, sec;

		/**
		 * Next scan time as UNIX time
		 */
		time_t nextScan;

		/**
		 * Auto start flag
		 */
		bool autoStart;

		/**
		 * Immediately
		 */
		bool immediately;
};

#endif




